package edu.berkeley.sbp.misc;
import edu.berkeley.sbp.util.*;
import edu.berkeley.sbp.*;
import java.util.*;
import java.io.*;

public class MetaGrammar extends ReflectiveWalker {

    public static Union make() throws Exception {
        return ((MetaGrammar)new MetaGrammar().walk(meta)).done();
    }
    public String toString() {
        StringBuffer ret = new StringBuffer();
        for(Union u : nt.values()) u.toString(ret);
        return ret.toString();
    }

    // FIXME
    private static HashSet<Element> dropAll = new HashSet<Element>();

    // Statics //////////////////////////////////////////////////////////////////////////////

    private static final Union SELF = new Union("()");

    public static Union epsilon = new Union("()");
    static { epsilon.add(Sequence.empty); }

    ////////////////////////////////////////////////////////////////////////////////

    private Union g;
    private HashMap<String,Union> nt;
    private int anon = 0;
    private String startSymbol;

    public MetaGrammar() { this("s"); }
    public MetaGrammar(String s) { done(s); }
    public Union done() { return done("s"); }
    public Union done(String str) {
        Union ret = g;
        g = new Union(str);
        startSymbol = str;
        nt = new HashMap<String,Union>();
        dropAll = new HashSet<Element>();
        nt.put(str, g);
        return ret;
    }


    // MetaGrammar //////////////////////////////////////////////////////////////////////////////

    public PreSequence _amp_(PreSequence p, Object[] o)        { return p.and(new PreSequence(o, null).buildSequence(null, true, false, true)); }
    public PreSequence _amp__tilde_(PreSequence p, Object[] o) { return p.not(new PreSequence(o, null).buildSequence(null, true, false, true)); }
    public Object      _star_(Element r)                                              { return Repeat.many0(r); }
    public Element     epsilon(Object o, Object b)                                    { return epsilon; }
    public Element     _rightparen_(Object e)                                         { return SELF; }
    public Element     _bang_(Element r)                                              { return r; }
    public Object      _caret_(String s)                                              { return new MyLift(s); }
    public Object      _leftparen__rightparen_()                                      { return epsilon; }
    public Union       nonTerminal(String s)                                          { return nonTerminal(s, null, false, false); }
    public Union       _colon__colon__equals_(String s, PreSequence[][] p)            { return nonTerminal(s, p, false, false); }
    public Union       _bang__colon__colon__equals_(String s, PreSequence[][] p)      { return nonTerminal(s, p, false, true); }
    public Union       _colon__colon__equals_(boolean q, String s, PreSequence[][] p) { return nonTerminal(s, p, false, q); }
    public Object      _leftparen_(PreSequence[][] p)                                 { return nonTerminal("anon"+(anon++), p, false, false); }
    public Union       nonTerminal(String str, PreSequence[][] s, boolean synthetic, boolean dropAll) {
        Union n = str.equals(startSymbol) ? g : nt.get(str);
        if (n == null) nt.put(str, n = new Union(str, synthetic));
        if (dropAll) this.dropAll.add(n);
        if (s==null) return n;
        HashSet<Sequence> seqs = new HashSet<Sequence>();
        for(int i=0; i<s.length; i++) {
            if (s[i]==null) continue;
            HashSet<Sequence> temp = new HashSet<Sequence>();
            for(PreSequence pre : s[i]) {
                pre.not.addAll(seqs);
                Sequence seq = pre.buildSequence(n, false, dropAll, false);
                temp.add(seq);
                n.add(seq);
            }
            seqs.addAll(temp);
        }
        return n;
    }

    public Object   _backslash__leftbrace_(String s)      { return SELF; }
    public Object   _leftbrace_(String s)                 { return SELF; }
    public Object   _plus_(final Element r) { return Repeat.many1(r); }

    public PreSequence rewrite(Object[] o)                                            { return new PreSequence(o, null); }
    public PreSequence _equals__greater_(Object[] o, String s)                        { return new PreSequence(o, s); }
    public PreSequence seq(Object[] o, Object sep, String tag) { return new PreSequence(sep==null ? o : _slash_(o, sep), tag); }
    public Object[] seq(Object[] o) { return o; }
    public Object[] _slash_(Object[] o, Object sep) {
        if (o.length <= 1) return o;
        Object[] ret = new Object[o.length * 2 - 1];
        for(int i=0; i<o.length; i++) {
            Object oi = o[i];
            ret[i*2]   = oi;
            if (i*2+1<ret.length) ret[i*2+1] = new MyDrop(sep);
        }
        return ret;
    }

    public Object _plus__slash_(final Element r, Element s) { return Repeat.many1(r, s); }
    public Object _star__slash_(final Element r, Element s) { return Repeat.many0(r, s); }
    public Object _star__star_(final Element r) { return Repeat.maximal(Repeat.many0(r)); }
    public Object _plus__plus_(final Element r) { return Repeat.maximal(Repeat.many1(r)); }
    public Element _question_(final Element r) { return Repeat.maybe(r); }
    public MetaGrammar gram(Object o, MetaGrammar g, Object o2) { return g; }
    public MetaGrammar  grammar(Object[] o) { return this; }
    public MetaGrammar grammar(Object o, Union[] u, Object x) { return this; }
    public char    _backslash_n() { return '\n'; }
    public char    _backslash_r() { return '\r'; }
    public Object    literal(String s) { return new MyDrop(CharToken.string(s)); }
    public Range     _minus_(char a, char b) { return new Range(a, b); }
    public Range     range(char a)         { return new Range(a, a); }
    public Element   _leftbracket_       (Range[] rr) { return ranges(true, rr); }
    public Element   _leftbracket__tilde_(Range[] rr) { return ranges(false,  rr); }
    public Element   ranges(boolean positive, Range[] rr) {
        Range.Set ret = positive ? new Range.Set() : new Range.Set(new Range(true, true));
        if (rr != null)
            for(Range r : rr)
                if (positive) ret.add(r);
                else          ret.remove(r);
        return CharToken.set(ret);
    }

    public static class MyLift {
        public final String s;
        public MyLift(String s) { this.s = s; }
    }
    public static class MyDrop {
        public final Object o;
        public MyDrop(Object o) { this.o = o; }
    }

    public static class PreSequence {
        public PreSequence not(Sequence s) { not.add(s); return this; }
        public PreSequence and(Sequence s) { and.add(s); return this; }
        public final HashSet<Sequence> and  = new HashSet<Sequence>();
        public final HashSet<Sequence> not  = new HashSet<Sequence>();
        public /*final*/ String tag;
        public final Object[] o;

        public PreSequence(Object[] o, String tag) { this.o = o; this.tag = tag; }
        boolean[] drops = null;
        public Union    buildUnion() {
            Union u = new Union("???");
            u.add(buildSequence(u));
            return u;
        }
        public Sequence buildSequence(Union u) { return buildSequence(u, false, false, false); }
        public Sequence buildSequence(Union u, boolean lame, boolean dropAll, boolean keeper) {
            for(Sequence s : and) u.add(s);
            for(Sequence s : not) u.add(s);
            HashSet<Sequence> set = new HashSet<Sequence>();
            Element[] o2 = o==null ? new Element[0] : new Element[o.length];
            if (o != null) {
                drops = new boolean[o.length];
                int j = 0;
                for(int i=0; i<o.length; i++) {
                    Object oi = o[i];
                    if (oi instanceof MyDrop)      { oi = ((MyDrop)oi).o; drops[j] = true; }

                    if (oi==SELF)                    o2[j] = u.new Subset("(("+u+"))", set);
                    else if (oi instanceof MyLift) { o2[j] = CharToken.string(tag = ((MyLift)oi).s); drops[j] = true; }
                    else                             o2[j] = (Element)oi;

                    if (MetaGrammar.dropAll.contains(o2[j])) drops[j] = true;
                    o2[j] = o2[j];
                    j++;
                }
            }
            Element[] expansion = o2;
            Sequence ret = dropAll || lame || keeper ? Sequence.drop(expansion, and, not, lame) : null;
            if (ret==null && tag!=null) ret = Sequence.rewritingSequence(tag, expansion, drops, and, not);
            if (ret==null) {
                int idx = -1;
                for(int i=0; i<expansion.length; i++) {
                    if (!drops[i]) idx = i;
                    else expansion[i] = expansion[i];
                }
                ret = idx==-1 ? Sequence.drop(expansion, and, not, false) : Sequence.singleton(expansion, idx, and, not);
            }
            set.add(ret);
            return ret;
        }
    }

    public static void main(String[] args) throws Exception {
        if (args.length != 2) {
            System.err.println("usage: java " + MetaGrammar.class.getName() + " grammarfile.g com.yourdomain.package.ClassName");
            System.exit(-1);
        }

        String className   = args[1].substring(args[1].lastIndexOf('.')+1);
        String packageName = args[1].substring(0, args[1].lastIndexOf('.'));
        String fileName    = packageName.replace('.', '/') + "/" + className + ".java";

        BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(fileName)));
        StringBuffer out = new StringBuffer();

        boolean skip = false;
        for(String s = br.readLine(); s != null; s = br.readLine()) {
            if (s.indexOf("DO NOT EDIT STUFF BELOW: IT IS AUTOMATICALLY GENERATED") != -1 && s.indexOf("\"")==-1) skip = true;
            if (s.indexOf("DO NOT EDIT STUFF ABOVE: IT IS AUTOMATICALLY GENERATED") != -1 && s.indexOf("\"")==-1) break;
            if (!skip) out.append(s+"\n");
        }

        out.append("\n        // DO NOT EDIT STUFF BELOW: IT IS AUTOMATICALLY GENERATED\n");
        new Parser(MetaGrammar.make(), CharToken.top()).parse1(new CharToken.Stream(new InputStreamReader(new FileInputStream(args[0])))).toJava(out);
        out.append("\n        // DO NOT EDIT STUFF ABOVE: IT IS AUTOMATICALLY GENERATED\n");

        for(String s = br.readLine(); s != null; s = br.readLine()) out.append(s+"\n");
        br.close();

        OutputStream os = new FileOutputStream(fileName);
        PrintWriter p = new PrintWriter(new OutputStreamWriter(os));
        p.println(out.toString());
        p.flush();
        os.close();
    }

    public static final Tree meta =






        // DO NOT EDIT STUFF BELOW: IT IS AUTOMATICALLY GENERATED
new Tree(null, "gram", new Tree[] { new Tree(null, null, new Tree[] { }),
        new Tree(null, "grammar", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "s", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "g", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "m", new Tree[] { }),
        new Tree(null, "m", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "r", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { new Tree(null, "g", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "m", new Tree[] { })})})})})}),
        new Tree(null, "!::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "**", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { })})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "g", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "m", new Tree[] { }),
        new Tree(null, "m", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "r", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "+/", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "r", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { new Tree(null, "g", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "m", new Tree[] { }),
        new Tree(null, "m", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "r", new Tree[] { })})})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "r", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})}),
        new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, ":", new Tree[] { }),
        new Tree(null, ":", new Tree[] { }),
        new Tree(null, "=", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "a", new Tree[] { }),
        new Tree(null, "l", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "v", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})}),
        new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "!", new Tree[] { }),
        new Tree(null, ":", new Tree[] { }),
        new Tree(null, ":", new Tree[] { }),
        new Tree(null, "=", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "a", new Tree[] { }),
        new Tree(null, "l", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "v", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, null, new Tree[] { })})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "a", new Tree[] { }),
        new Tree(null, "l", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "v", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "+/", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "A", new Tree[] { }),
        new Tree(null, "l", new Tree[] { }),
        new Tree(null, "t", new Tree[] { })})}),
        new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, ">", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "A", new Tree[] { }),
        new Tree(null, "l", new Tree[] { }),
        new Tree(null, "t", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "+/", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "c", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "j", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "|", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "c", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "j", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "r", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "w", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "r", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "w", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "&", new Tree[] { })})}),
        new Tree(null, "*/", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "r", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "w", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "&", new Tree[] { }),
        new Tree(null, "~", new Tree[] { })})}),
        new Tree(null, "*/", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, null, new Tree[] { })})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "s", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "*/", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})})}),
        new Tree(null, "?", new Tree[] { new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "/", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})})}),
        new Tree(null, "?", new Tree[] { new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "=", new Tree[] { }),
        new Tree(null, ">", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { new Tree(null, "s", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "q", new Tree[] { })})})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "r", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "w", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "s", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "c", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "[~", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "range", new Tree[] { new Tree(null, "-", new Tree[] { })}),
        new Tree(null, "range", new Tree[] { new Tree(null, "]", new Tree[] { })}),
        new Tree(null, "range", new Tree[] { new Tree(null, "\\", new Tree[] { })}),
        new Tree(null, "range", new Tree[] { new Tree(null, "~", new Tree[] { })})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "s", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "p", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "g", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "c", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "g", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "c", new Tree[] { })})}),
        new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "-", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "c", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { new Tree(null, "n", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "T", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "m", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "l", new Tree[] { })})}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "[", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "range", new Tree[] { new Tree(null, "(", new Tree[] { })})})}),
        new Tree(null, "[", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "range", new Tree[] { new Tree(null, ")", new Tree[] { })})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "p", new Tree[] { }),
        new Tree(null, "s", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "l", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "n", new Tree[] { })})}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "{", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "a", new Tree[] { }),
        new Tree(null, "l", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "v", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "}", new Tree[] { })})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "[", new Tree[] { })})}),
        new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "*", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "g", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "]", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "[", new Tree[] { }),
        new Tree(null, "~", new Tree[] { })})}),
        new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "*", new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "g", new Tree[] { }),
        new Tree(null, "e", new Tree[] { })})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "]", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "^", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "!", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "*", new Tree[] { }),
        new Tree(null, "*", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})}),
        new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "*", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "*", new Tree[] { }),
        new Tree(null, "/", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "+", new Tree[] { }),
        new Tree(null, "+", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})}),
        new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "+", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "+", new Tree[] { }),
        new Tree(null, "/", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { })})}),
        new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "?", new Tree[] { })})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { new Tree(null, "l", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "l", new Tree[] { })})}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "(", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})}),
        new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, ")", new Tree[] { })})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, null, new Tree[] { })})}),
        new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "^", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "(", new Tree[] { })})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "a", new Tree[] { }),
        new Tree(null, "l", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "n", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "i", new Tree[] { }),
        new Tree(null, "v", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, ")", new Tree[] { })})})}),
        new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "s", new Tree[] { })})}),
        new Tree(null, null, new Tree[] { })})})})}),
        new Tree(null, "!::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, " ", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "/", new Tree[] { }),
        new Tree(null, "/", new Tree[] { })})}),
        new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "*", new Tree[] { new Tree(null, "[~", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "range", new Tree[] { new Tree(null, "\n", new Tree[] { })})})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\n", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\n", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\r", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "w", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "r", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "++", new Tree[] { new Tree(null, "[", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "-", new Tree[] { new Tree(null, "a", new Tree[] { }),
        new Tree(null, "z", new Tree[] { })}),
        new Tree(null, "-", new Tree[] { new Tree(null, "A", new Tree[] { }),
        new Tree(null, "Z", new Tree[] { })}),
        new Tree(null, "-", new Tree[] { new Tree(null, "0", new Tree[] { }),
        new Tree(null, "9", new Tree[] { })}),
        new Tree(null, "range", new Tree[] { new Tree(null, "_", new Tree[] { })})})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "q", new Tree[] { }),
        new Tree(null, "u", new Tree[] { }),
        new Tree(null, "o", new Tree[] { }),
        new Tree(null, "t", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\"", new Tree[] { })})}),
        new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "+", new Tree[] { new Tree(null, "(", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "[~", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "range", new Tree[] { new Tree(null, "\"", new Tree[] { })}),
        new Tree(null, "range", new Tree[] { new Tree(null, "\\", new Tree[] { })})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "nonTerminal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "s", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "p", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})}),
        new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\"", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\"", new Tree[] { }),
        new Tree(null, "\"", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, "", new Tree[] { })})})})}),
        new Tree(null, "::=", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "e", new Tree[] { }),
        new Tree(null, "s", new Tree[] { }),
        new Tree(null, "c", new Tree[] { }),
        new Tree(null, "a", new Tree[] { }),
        new Tree(null, "p", new Tree[] { }),
        new Tree(null, "e", new Tree[] { }),
        new Tree(null, "d", new Tree[] { })}),
        new Tree(null, null, new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\\", new Tree[] { }),
        new Tree(null, "n", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { new Tree(null, "\n", new Tree[] { })})}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\\", new Tree[] { }),
        new Tree(null, "r", new Tree[] { })})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { new Tree(null, "\r", new Tree[] { })})}),
        new Tree(null, "seq", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "literal", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "\\", new Tree[] { })})}),
        new Tree(null, "[~", new Tree[] { new Tree(null, null, new Tree[] { new Tree(null, "range", new Tree[] { new Tree(null, "n", new Tree[] { })}),
        new Tree(null, "range", new Tree[] { new Tree(null, "r", new Tree[] { })})})})}),
        new Tree(null, null, new Tree[] { }),
        new Tree(null, null, new Tree[] { })})})})})})}),
        new Tree(null, null, new Tree[] { })})
        // DO NOT EDIT STUFF ABOVE: IT IS AUTOMATICALLY GENERATED
        ;
}
































